﻿using Hotline.Caching.Interfaces;
using Hotline.Settings.TimeLimits;
using Hotline.Share.Dtos.Settings;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Settings;
using MapsterMapper;
using XF.Domain.Exceptions;
using XF.Domain.Repository;
using XF.Utility.EnumExtensions;

namespace Hotline.Settings.TimeLimitDomain;

/// <summary>
/// 过期时间处理基类
/// </summary>
public abstract class ExpireTimeLimitBase 
{
    private readonly ISystemSettingCacheManager _systemSettingCacheManager;
    private readonly IRepository<TimeLimitSetting> _timeLimitSettingRepository;
    private readonly IExpireTimeHandler _expireTimeHandler;
    private readonly IMapper _mapper;
    private readonly IRepository<SystemSetting> _systemSettingRepository;
    private readonly IDaySettingRepository _daySettingRepository;

    public ExpireTimeLimitBase(ISystemSettingCacheManager systemSettingCacheManager, IRepository<TimeLimitSetting> timeLimitSettingRepository, IExpireTimeHandler expireTimeHandler, IMapper mapper, IRepository<SystemSetting> systemSettingRepository, IDaySettingRepository daySettingRepository)
    {
        _systemSettingCacheManager = systemSettingCacheManager;
        _timeLimitSettingRepository = timeLimitSettingRepository;
        _expireTimeHandler = expireTimeHandler;
        _mapper = mapper;
        _systemSettingRepository = systemSettingRepository;
        _daySettingRepository = daySettingRepository;
    }

    public virtual async Task<int> CalcWorkTime(DateTime beginTime, DateTime endTime, bool isCenter)
    {
        var workTime = _systemSettingRepository.Get(x => x.Code == SettingConstants.WorkTime);
        return await _expireTimeHandler.CalcWorkTimeAsync(beginTime, endTime, workTime.SettingValue, isCenter);
    }

    public virtual async Task<int> CalcWorkTimeEx(DateTime beginTime, DateTime endTime, bool isCenter)
    {
        var workTime = _systemSettingRepository.Get(x => x.Code == SettingConstants.WorkTimeOrg);
        return await _expireTimeHandler.CalcWorkTimeExAsync(beginTime, endTime, workTime.SettingValue, isCenter);
    }

    /// <summary>
    /// 判断当前时间是否是工作日 返回一个工作日
    /// </summary>
    /// <param name="date"></param>
    /// <returns></returns>
    public virtual async Task<DateTime> WorkDay(DateTime date)
    {
        var workTime = _systemSettingRepository.Get(x => x.Code == SettingConstants.WorkTime);
        var (WorkBeginTime, WorkEndTime) = GetWorkTime(DateTime.Now, workTime.SettingValue);
        if (await IsWorkDay(date))
        {
            if (date < WorkBeginTime || date > WorkEndTime)
            {
                date = DateTime.Parse(date.ToShortDateString() + " " + workTime.SettingValue[0] + ":00");
            }
        }
        else
        {
            while (await NotWorkDay(date))
            {
                date = date.AddDays(1);
            }
            if (date < WorkBeginTime || date > WorkEndTime)
            {
                date = DateTime.Parse(date.ToShortDateString() + " " + workTime.SettingValue[0] + ":00");
            }
        }
        return date;
    }

    public async Task<DateTime?> CalcWorkTimeReduce(DateTime now, int timeValue)
    {
        return await _expireTimeHandler.CalcWorkTimeReduceAsync(now, timeValue);
    }

    /// <summary>
    /// 获取办理时限配置
    /// </summary>
    /// <param name="code"></param>
    /// <returns></returns>
    public virtual TimeConfig GetOrderTimeLimitConfig(string? code = null)
    {
        if (string.IsNullOrEmpty(code))
        {
            return new TimeConfig(
                int.Parse(_systemSettingCacheManager.GetSetting(SettingConstants.BackCenterTimeSetting).SettingValue[0]),
                ETimeType.WorkDay);
        }
        else
        {
            var timeSetting = _timeLimitSettingRepository.Queryable().First(x => x.BusCode == code);
            return new TimeConfig(timeSetting.TimeValue, timeSetting.TimeType);
        }
    }

    /// <summary>
    /// 计算期满时间
    /// </summary>
    public virtual async Task<ExpiredTimeWithConfig?> CalcExpiredTime(DateTime beginTime, EFlowDirection flowDirection, string? acceptTypeCode)
    {
        if (flowDirection is EFlowDirection.CenterToOrg && string.IsNullOrEmpty(acceptTypeCode))
            throw new UserFriendlyException("中心派至部门的工单期满时间需受理类型参数");
        var timeConfig = flowDirection switch
        {
            EFlowDirection.CenterToOrg => GetOrderTimeLimitConfig(acceptTypeCode),
            EFlowDirection.OrgToCenter => GetOrderTimeLimitConfig(),
            EFlowDirection.CenterToCenter => GetOrderTimeLimitConfig(acceptTypeCode),
            _ => throw new ArgumentOutOfRangeException(nameof(flowDirection), flowDirection, null)
        };

        return await CalcEndTime(beginTime, timeConfig, acceptTypeCode);
    }

    public virtual async Task<ExpiredTimeWithConfig?> CalcEndTime(DateTime beginTime, TimeConfig timeConfig, string? busCode)
    {
        var setting = _timeLimitSettingRepository.Queryable().First(x => x.BusCode == busCode);
        if (setting == null) return null;

        var workTime = _systemSettingRepository.Get(x => x.Code == SettingConstants.WorkTime);
        var result = await _expireTimeHandler.CalcEndTimeAsync(beginTime, timeConfig.TimeType, workTime.SettingValue, timeConfig.Count, setting.Percentage, setting.PercentageOne);
        var expiredTimeWithConfig = _mapper.Map<ExpiredTimeWithConfig>(timeConfig);
        expiredTimeWithConfig.ExpiredTime = result.EndTime;
        expiredTimeWithConfig.NearlyExpiredTime = result.NearlyExpiredTime;
        expiredTimeWithConfig.NearlyExpiredTimeOne = result.NearlyExpiredTimeOne;
        return expiredTimeWithConfig;
    }

    /// <summary>
    /// 计算工作日
    /// </summary>
    /// <param name="beginTime"></param>
    /// <param name="busCode"></param>
    /// <param name="isCenter"></param>
    /// <returns></returns>
    public virtual async Task<ExpiredTimeWithConfig?> CalcEndTime(DateTime beginTime, string busCode)
    {
        var setting = _timeLimitSettingRepository.Queryable().First(x => x.BusCode == busCode);
        if (setting == null) return null;

        var workTime = _systemSettingRepository.Get(x => x.Code == SettingConstants.WorkTime);
        var result = await _expireTimeHandler.CalcEndTimeAsync(beginTime, setting.TimeType, workTime.SettingValue, setting.TimeValue, setting.Percentage, setting.PercentageOne);
        var expiredTimeWithConfig = new ExpiredTimeWithConfig
        {
            TimeText = setting.TimeValue + "个" + setting.TimeType.GetDescription(),
            TimeType = setting.TimeType,
            Count = setting.TimeValue,
            ExpiredTime = result.EndTime,
            NearlyExpiredTime = result.NearlyExpiredTime,
            NearlyExpiredTimeOne = result.NearlyExpiredTimeOne
        };
        return expiredTimeWithConfig;
    }

    public virtual async Task<TimeResult?> CalcEndTime(DateTime beginTime, ETimeType timeType, int timeValue, string busCode)
    {
        var workTime = _systemSettingRepository.Get(x => x.Code == SettingConstants.WorkTime);
        var setting = _timeLimitSettingRepository.Queryable().First(x => x.BusCode == busCode);
        if (setting == null) return null;
        return await _expireTimeHandler.CalcEndTimeAsync(beginTime, timeType, workTime.SettingValue, timeValue, setting.Percentage, setting.PercentageOne);
    }

    public async Task<TimeResult> CalcEndTime(DateTime beginTime, ETimeType timeType, int timeValue, int Percentage, int PercentageOne)
    { 
        var workTime = _systemSettingRepository.Get(x => x.Code == SettingConstants.WorkTime);
        return await _expireTimeHandler.CalcEndTimeAsync(beginTime, timeType, workTime.SettingValue, timeValue, Percentage, PercentageOne);
    }

    /// <summary>
    /// 计算工作时间小时
    /// </summary>
    /// <param name="beginTime"></param>
    /// <param name="endTime"></param>
    /// <param name="isCenter"></param>
    /// <returns></returns>
    public virtual async Task<decimal> CalcWorkTimeToHour(DateTime beginTime, DateTime endTime, bool isCenter)
    {
        var workTime = _systemSettingRepository.Get(x => x.Code == SettingConstants.WorkTime);
        var min = await _expireTimeHandler.CalcWorkTimeAsync(beginTime, endTime, workTime.SettingValue, isCenter);
        if (min == 0) return 0;
        if (isCenter)
        {
            return Math.Round((decimal)min / 60 / 60 / 24, 2);
        }
        else
        {
            if (workTime == null) return 0;
            return Math.Round((decimal)min / 60 / 60, 2);
        }
    }

    /// <summary>
    /// 计算工作时间天数
    /// </summary>
    /// <param name="beginTime"></param>
    /// <param name="endTime"></param>
    /// <param name="isCenter"></param>
    /// <returns></returns>
    public virtual async Task<decimal> CalcWorkTimeToDecimal(DateTime beginTime, DateTime endTime, bool isCenter)
    {
        var workTime = _systemSettingRepository.Get(x => x.Code == SettingConstants.WorkTime);
        var min = await _expireTimeHandler.CalcWorkTimeAsync(beginTime, endTime, workTime.SettingValue, isCenter);
        if (min == 0) return 0;
        if (isCenter) return Math.Round((decimal)min / 60 / 60 / 24, 2);

        if (workTime == null) return 0;

        var (workBeginTime, workEndTime) = GetWorkTime(beginTime, workTime.SettingValue);
        var minuteSpan = new TimeSpan(workEndTime.Ticks - workBeginTime.Ticks);

        //总时差分钟数
        int minutes = (int)minuteSpan.TotalMinutes;
        return Math.Round(((decimal)min / 60) / minutes, 2);
    }

    public virtual async Task<TimeResult?> CalcEndTime(EFlowDirection flowDirection, int Percentage, int PercentageOne, ETimeType? timeType = null, int? timeValue = null)
    {
        var workTime = _systemSettingRepository.Get(x => x.Code == SettingConstants.WorkTime);
        switch (flowDirection)
        {
            case EFlowDirection.CenterToOrg:
                if (!timeType.HasValue || !timeValue.HasValue)
                    throw new UserFriendlyException("无效参数");
                return await _expireTimeHandler.CalcEndTimeAsync(DateTime.Now, timeType.Value, workTime.SettingValue, timeValue.Value, Percentage, PercentageOne);
            case EFlowDirection.OrgToCenter:
                //todo 根据配置
                return await _expireTimeHandler.CalcEndTimeAsync(DateTime.Now, ETimeType.Day, workTime.SettingValue, 1, Percentage, PercentageOne);
            default:
                throw new ArgumentOutOfRangeException(nameof(flowDirection), flowDirection, null);
        }
    }

    /// <summary>
    /// 是工作日
    /// 根据数据库的工作日和周末时间判断
    /// </summary>
    /// <param name="date"></param>
    /// <returns></returns>
    private async Task<bool> IsWorkDay(DateTime date) =>
        await _daySettingRepository.IsWorkDay(date);

    /// <summary>
    /// 非工作日
    /// 根据数据库的工作日和周末时间判断
    /// </summary>
    /// <param name="date"></param>
    /// <returns></returns>
    private async Task<bool> NotWorkDay(DateTime date)
        => !await IsWorkDay(date);

    /// <summary>
    /// 获取给定日期的工作时间
    /// </summary>
    /// <param name="date"></param>
    /// <param name="workTime"></param>
    /// <returns></returns>
    private static (DateTime startTime, DateTime endTime) GetWorkTime(DateTime date, List<string> workTime)
    {
        var startTime = DateTime.Parse(date.ToShortDateString() + " " + workTime[0] + ":00");
        var endTime = DateTime.Parse(date.ToShortDateString() + " " + workTime[1] + ":00");
        return (startTime, endTime);
    }

}
